var editManager = function() {
	var _editdata;
	var _uas;
	
	var _enablers = {
		'refererEnabled': 'referer',
		'useragentEnabled': 'useragent',
		'languageEnabled': 'language',
	};
	var _configs = {
		'referer': {
			'external': 'fldRefererExt',
			'custom': 'fldRefererCustomText',
			'mode': 'fldReferer',
		},
		'useragent': {
			'mode': 'fldUseragent',
			'custom': 'fldUseragentCustom',
		},
		'language': {
			'mode': 'fldLanguageOnly',
			'lang': 'fldLanguageLanguage',
			'custom': 'fldLanguageCustom',
		}
	};
	var _subenablers = {
		'fldReferer': _updateReferer,
		'fldUseragent': _updateUseragent,
		'fldLanguageLanguage': _updateLanguage,
	};
	
	
	function getString(stringID) {
		return document.getElementById('strings').getString(stringID);
	}

	function _loadPresets() {
		_uas = headercontrolPrefs.getUseragents();

		fillItems('popUseragent', _uas.sortkey);
		fillItems('popLanguage', getBrowserLanguages());
	}

	// fills <menupopup> (with given popupId) with items
	function fillItems(popupId, items) {
		var popup = document.getElementById(popupId)

		// We need to remove dummy menuitem. Dummy is needed in Linux in order for a popup not to be a thin line.
		// TODO: find out if dummies are still really required.
		popup.removeChild(popup.querySelector('#dummy'));

		var insertBefore = popup.getElementsByTagName('menuseparator')[0];

		for(var item in items) {
			item = items[item];
			var el = document.createElement('menuitem');
			el.setAttribute("label", item.label);
			el.setAttribute("value", item.id);
			popup.insertBefore(el, insertBefore);
		}
	}

	function getBrowserLanguages() {
		var popup = document.createElement('menupopup');
		popup.id = 'availableLanguagesPopup' // copy from browser.jar
		document.getElementById('fldLanguageLanguage').appendChild(popup);

		// languages from browser: this method seeks for 'availableLanguagesPopup' element and fills it with items
		gLanguagesDialog._loadAvailableLanguages();

		var langs = [];
		for (var i = 0; i < popup.childNodes.length; i++) {
			var el = popup.childNodes[i];
			el.value = el.id;
			langs.push({ id: el.id, label: el.label });
		}

		popup.parentNode.removeChild(popup);
		return langs;
	}
	
	function _updateEnable(enabler) {
		var category = _enablers[enabler];
		var fields = _configs[category];
		var disable = document.getElementById(enabler).value != 'on';
		var afterrun;
	
		for each (var onefield in fields) {
			document.getElementById(onefield).disabled = disable;
			if (_subenablers[onefield] != undefined) {
				afterrun = onefield;
			}
		}
		
		if (!disable && (afterrun != undefined)) {
			_subenablers[afterrun](document.getElementById(afterrun));
		}
	}
	
	function _updateReferer(field) {
		document.getElementById('fldRefererCustomText').disabled =
			(field.selectedItem == null || field.selectedItem.value != 'custom');	
	}
	
	function _updateUseragent(field) {
		_updateCustomField(field, 'fldUseragentCustom', function(id) { return _uas.useragents[_uas.byID[id]].header });
	}

	function _updateLanguage(field) {
		_updateCustomField(field, 'fldLanguageCustom');
	}

	function _updateCustomField(field, customFieldId, mapper) {
		mapper = mapper || function(id) { return id; };
		var customField = document.getElementById(customFieldId);

		customField.disabled =
			(field.disabled || field.selectedItem == null || field.selectedItem.value != 'custom');
		if (field.selectedItem != null
		 && field.selectedItem.value != null
		 && field.selectedItem.value != ''
		 && field.selectedItem.value != 'custom') {
			customField.value = mapper(field.selectedItem.value);
		}
	}
	
	function _loadConfig(category) {
		for (var configattr in _configs[category]) {
			try {
				var obj = document.getElementById(_configs[category][configattr]);
				var value = _editdata.data[category][configattr];
				
				switch (obj.tagName) {
					case 'checkbox':
						obj.checked = (value != undefined ? value : false);
						break;
					case 'radiogroup':
					case 'menulist':
						if (value != undefined) {
							for (var i=0; i < obj.itemCount; i++) {
								if (obj.getItemAtIndex(i).value == value) {
									obj.selectedIndex = i;
									break;
								}
							}
						}
						break;
					default:
						obj.value = (value != undefined ? value : "");
						break;
				}
			}
			catch(e) {
				//alert( "leave out this one:" + category );
			}
		}
	}
	
	function _saveConfig(category) {
		for (var configattr in _configs[category]) {
			var obj = document.getElementById(_configs[category][configattr]);
			
			switch (obj.tagName) {
				case 'checkbox':
					if( obj.checked != null )
						_editdata.data[category][configattr] = obj.checked;
					else
						_editdata.data[category][configattr] = false;
					break;
				case 'radiogroup':
				case 'menulist':
					if( obj.selectedItem.value != null )
						_editdata.data[category][configattr] = obj.selectedItem.value;
					else
						_editdata.data[category][configattr] = "";
					break;
				default:
					if( obj.value != null )
						_editdata.data[category][configattr] = obj.value;
					else
						_editdata.data[category][configattr] = "";
					break;
			}
		}
	}
	
	return {
		onLoad : function() {
			_editdata = window.arguments[0];

			_loadPresets(); // populate "static" GUI
			
			document.getElementById('fldSuffix').value = _editdata.suffix;

			// disable for "default" site
			if (_editdata.suffix == "") {
				document.getElementById('fldSuffix').disabled = true;
			}
			
			for (var enabler in _enablers) {
				var ctrl = document.getElementById(enabler);

				// disable "inherit" for "default" site
				if (_editdata.suffix == "") {
					var arr = Array.prototype.slice.call(ctrl.getElementsByTagName('radio'));
					arr.filter(function(e) { return e.value == 'inherit' })[0].disabled = true;
				}

				var category = _enablers[enabler];
				ctrl.value = _editdata.data[category].enabled;
				_loadConfig(category);
				_updateEnable(enabler);
			}
			
			for (var subenabler in _subenablers) {
				_subenablers[subenabler](document.getElementById(subenabler));
			}
		},
		
		onOK : function() {
			try {
				for (var enabler in _enablers) {
					var category = _enablers[enabler];
					var ctrl = document.getElementById(enabler);
					_editdata.data[category] = {};
					_editdata.data[category].enabled = ctrl.value;
					_saveConfig(category);
				}
			}
			catch(e) {
			}
			window.arguments[0].suffix = document.getElementById('fldSuffix').value;
			window.arguments[0].data = _editdata.data;
			window.arguments[0].save = true;
			return true;
		},

		updateEnable: function(field) {	_updateEnable(field.id); },
		updateReferer: function(field) { _updateReferer(field); },
		updateUseragent: _updateUseragent,
		updateLanguage: _updateLanguage,
	};
}();
