Components.utils.import("resource://gre/modules/AddonManager.jsm");

var optionsManager = function() {
	var _suffixes = null;
	var _keys = null;
	var _tree = null;

	function getString( stringID )
	{
		return document.getElementById( 'strings' ).getString( stringID );
	}

	function stringCompare(a, b) {
		a = a.split( '.' );
		b = b.split( '.' );
		var t = 0;
		
		while ( !t && a.length && b.length ) {
			a.pop();
			b.pop();
			t = (b<a)-(a<b);
		}
		
		return t + (a.length > 0) - (b.length > 0)
	}
	
	function updateConfiguration( suff ) {
		_suffixes = suff;
		
		_keys = new Array();
		for (var s in _suffixes) {
			_keys.push(s);
		}
		_keys = _keys.sort(stringCompare);
		
		document.getElementById( 'treeSuffixes' ).boxObject.invalidate = true;
		
		_tree = document.getElementById( 'treeSuffixes' );
		_tree.view = view;
	}

	function _onSelected() {
		var selection = view.selection;
		document.getElementById("btnEdit").disabled = !(selection && (selection.count == 1));
		document.getElementById("btnRemove").disabled = !selection || (selection.count < 1) || (_keys[selection.currentIndex] == "");
	}
	
	function doEdit(newSuffix, garbageColect)
	{
		var oldindex = view.selection.currentIndex;
		
		var editing = { 
			suffix: newSuffix,
			data: _suffixes[newSuffix],
			save: false,
		};
		openDialog("chrome://headercontrol/content/edit.xul", "", "centerscreen,chrome,modal,resizable=no", editing);
	
		if (editing.save == false)
		{
			// Remove temporary suffix
			if ( garbageColect == true ) {
				delete _suffixes[newSuffix];
			}
		}
		else
		{
			_suffixes[editing.suffix] = editing.data;
			
			if (editing.suffix != newSuffix || garbageColect == true)
			{
				delete _suffixes[newSuffix];
				//updateConfiguration( _suffixes );
				//view.selection.select();
			}
		}
	}
		
	var view = function() {
		var atomNullSuffix = Components.classes["@mozilla.org/atom-service;1"]
			.getService(Components.interfaces.nsIAtomService).getAtom("nullsuffix");
		var atomSuffix = Components.classes["@mozilla.org/atom-service;1"]
			.getService(Components.interfaces.nsIAtomService).getAtom("suffix");

		var colMap = {
			'colReferer': 'referer',
			'colUA': 'useragent',
			'colLang': 'language',
		};

		return {
			// begin nsITreeView
			get rowCount() 
			{
				return _keys.length; 
			},

			getCellText: function( aRow, aColumn )
			{
				if (aColumn.id == "colSuffix") {
					return _keys[aRow] ? _keys[aRow] : "- default -"; 
				}
				else {
					return "";
				}
			},

			getImageSrc: function(aRow, aColumn) {
				var prop = colMap[aColumn.id];
				return prop !== undefined && headercontrolPrefs.isEnabled(_suffixes, _keys[aRow], prop) ? 
					"chrome://headercontrol/skin/actions/present.png" : "";
			},

			getCellProperties: function(aRow, aColumn, aProperty)
			{
				if (aColumn.id == "colSuffix") {
					if (_keys[aRow] == "") {
						aProperty.AppendElement(atomNullSuffix);
					}
					else {
						aProperty.AppendElement(atomSuffix);
					}
				}
			},

			isSeparator: function(aIndex) { return false; },
			isSorted: function() { return false; },
			isContainer: function(aIndex) { return false; },
			setTree: function(aTree){},
			getProgressMode: function(aRow, aColumn) {},
			getCellValue: function(aRow, aColumn) {},
			cycleHeader: function(aColId, aElt) {},
			getRowProperties: function(aRow, aProperty) {},
			getColumnProperties: function(aColumn, aColumnElement, aProperty) {},
			// end nsITreeview
		};
	}();

	return {
		onSelected: function()
		{
			_onSelected();
		},

		onLoad: function()
		{
			updateConfiguration( headercontrolPrefs.getSuffixConf() );
			
			this.onSelected();
			if (headercontrolPrefs.getStatus()) {
				document.getElementById( 'checkEnabled' ).checked = 'true';
			}
		},
	
		onOK: function()
		{
			headercontrolPrefs.setSuffixConf( _suffixes );
			headercontrolPrefs.setStatus(document.getElementById( 'checkEnabled' ).checked);
			return true;
		},
	
		onKeyPress: function(aEvent)
		{
			if (aEvent.keyCode == 46)	/* Delete */
			{
				if (!document.getElementById("btnRemove").disabled)
					this.onRemove();
			}
		},
	
		onDblClick: function(aEvent)
		{
			if (aEvent.target._lastSelectedRow != -1)
				doEdit(_keys[aEvent.target._lastSelectedRow], false);
			else
				this.onAdd();
		},

		onAbout: function()
		{
			AddonManager.getAddonByID(
				'headercontrolrevived@torvin',
				function(addon) {
					openDialog("chrome://mozapps/content/extensions/about.xul", "", "chrome,modal", addon);
				}
			);
		},
		
		onImport: function(){ alert( "not yet implemented" );	},
		onExport: function(){ alert( "not yet implemented" );	},
		
		onAdd: function()
		{
			var newSuffix = "example.org";
			
			_suffixes[newSuffix] = {
				referer: { enabled: "inherit" },
				useragent: { enabled: "inherit" },
				language: { enabled: "inherit" },
        	};
			doEdit(newSuffix, true);
			
			updateConfiguration(_suffixes);
		},

		onEdit: function()
		{
			var selection = view.selection;
			doEdit( _keys[selection.currentIndex], false );
			
			updateConfiguration(_suffixes);
		},

		onRemove: function()
		{
			var selection = view.selection;
			var oldindex = selection.currentIndex;
			
			selection.selectEventsSuppressed = true;
			
			var rangeCount = selection.getRangeCount();
			for (var range = 0; range < rangeCount; range++)
			{
				var min = new Object(), max = new Object();
				selection.getRangeAt(range, min, max);
				for (var i = min.value; i <= max.value; i++)
				{
					delete _suffixes[_keys[i]];
				}
			}
			updateConfiguration(_suffixes);

			selection.selectEventsSuppressed = false;
			
//			if (oldindex > view.rowCount - 1)
//				oldindex = view.rowCount - 1;
//			view.selection.select(oldIndex);
//			_tree.treeBoxObject.ensureRowIsVisible(oldIndex);
		},

		onRemoveAll: function()
		{
			if (!window.confirm(getString("ConfirmRemoveAll")))
				return;

			headercontrolPrefs.clearSuffixConf();
			updateConfiguration(headercontrolPrefs.getSuffixConf());
			
			var selection = view.selection;
			
			if (selection)
				selection.clearSelection();
		},
	};
}();

	
	
