var headercontrolPrefs = function() {

	var prefbranch = Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService)
			.getBranch("extensions.headercontrol.");

	if (typeof(JSON) == "undefined")
	{
		Components.utils.import("resource://gre/modules/JSON.jsm");
		JSON.parse = JSON.fromString;
		JSON.stringify = JSON.toString;
	}

	// upgrading old boolean 'enabled' field with new tri-state one
	var upgradeEnabled = function(enabled, isDefault) {
		if (enabled === true)
			return 'on';
		else if (enabled === false || enabled === undefined)
			return isDefault ? 'off' : 'inherit';
		else
			return enabled;
	}

	// upgrading old boolean 'enabled' field with new tri-state one
	var upgradeConfig = function(cfg) {
		var sections = [ "referer", "useragent", "language" ];
		if (cfg[""] == undefined)
			cfg[""] = {};
		for (var suff in cfg) {
			var site = cfg[suff];
			var newSite = {};
			for(var sectionName in sections) {
				sectionName = sections[sectionName];
				var section = site[sectionName];
				if (section === undefined)
					continue;
				section.enabled = upgradeEnabled(section.enabled, suff == "");
				newSite[sectionName] = section;
			}
			cfg[suff] = newSite;
		}
		return cfg;
	}

	var triToBool = function(val, parentVal) {
		if (val == 'on')
			return true;
		if (val == 'off')
			return false;
		if (val == 'inherit' && parentVal != undefined)
			return triToBool(parentVal);
		return false;
	}

	return {
		setStatus: function(enabled) {
			prefbranch.setBoolPref( 'enabled', enabled );
		},
		
		getStatus: function() {
			return prefbranch.getBoolPref( 'enabled' );	
		},
		
		getSuffixConf: function() {
			var cfg = JSON.parse( prefbranch.getCharPref( "conf" ) );
			return upgradeConfig(cfg);
		},

		isEnabled: function(cfg, site, section) {
			return triToBool(cfg[site][section].enabled, cfg[""][section].enabled);
		},
		
		setSuffixConf : function(suff) {
			prefbranch.setCharPref( "conf", JSON.stringify(suff) );
		},
		
		clearSuffixConf : function() {
			prefbranch.resetBranch();
		},
		
		getUseragents: function() {
			var t = {};
			var childs = {};
			var res = {
				useragents: [],
				sortkey: [],
				byID: {},
			};
			
			childs = prefbranch.getChildList( "useragent.", t );
			t = {};
			for each (var key in childs) {
				var kparts = key.split('.');

				if (t[kparts[1]] == undefined) {
					t[kparts[1]] = res.useragents.length;
					res.useragents.push( {} );
				}
	
				res.useragents[t[kparts[1]]][kparts[2]] = prefbranch.getCharPref( key );
				
				if (kparts[2] == "label") {
					res.sortkey.push( { 'label': res.useragents[t[kparts[1]]][kparts[2]], 'prefindex': t[kparts[1]] } );
				}
			}
			
			for (var i=0; i<res.sortkey.length; i++) {
				res.sortkey[i].id = res.useragents[res.sortkey[i].prefindex].id;
				res.byID[res.sortkey[i].id] = res.sortkey[i].prefindex;
			}
			
			function innercompare( a, b ) {
				return (a.label > b.label) - (b.label > a.label);
			}
			res.sortkey = res.sortkey.sort(innercompare);
			
			return res;
		}
	}
}();

var LOG = function() {
	var aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].
	     getService(Components.interfaces.nsIConsoleService);

	return function(msg) {
		aConsoleService.logStringMessage(msg);
	}
}();

