Components.utils.import("resource://gre/modules/XPCOMUtils.jsm"); 

function HeaderControl() { }

var headercontrolEngine = function() {
	var _enabled = true;
	var _conf = {};

	var _pref;

	if (typeof(JSON) == "undefined")
	{
		Components.utils.import("resource://gre/modules/JSON.jsm");
		JSON.parse = JSON.fromString;
		JSON.stringify = JSON.toString;
	}
	
	function _readConf() {
		var lconf = JSON.parse(_pref.getCharPref( "conf" ));
		
		_conf = {};
		
		for (var suffix in lconf) {
			var tsuffix = suffix.split(".");
			tsuffix = tsuffix.reverse();
			tsuffix.unshift( "" );
			tsuffix = tsuffix.join(".");
			var conf = lconf[suffix];
			for(var sectionName in conf) {
				var section = conf[sectionName];
				if (suffix != "" && section.enabled == 'inherit') {
					conf[sectionName] = lconf[""][sectionName];
				} else {
					section.enabled = section.enabled == 'on';
				}
			}
			_conf[tsuffix] = conf;
		}
		_conf[""] = _conf["."];
		delete _conf["."];
	}

	function _adjustReferer(channel, suffix) {
		try {
			var nreferer;

			if (_conf[suffix].referer.external && 
				((channel.referrer != null) && 
				 (channel.URI.host == channel.referrer.host)
				)) {
				return false;
			}

			switch (_conf[suffix].referer.mode) {
				case "block":
					nreferer = "";
					break;
				case "root":
					nreferer = channel.URI.scheme + "://" + channel.URI.hostPort + "/";
					break;
				default:
					nreferer = 	_conf[suffix].referer.custom;
					break;
			}

			channel.setRequestHeader("Referer", nreferer, false);
			if (channel.referrer)
				channel.referrer.spec = nreferer;

			return true;
		} catch (ex) {
			dump("_adjustReferer: " + ex);
		}
		return false;
	}

	function _adjustUseragent(channel, suffix) {
		try {
			var nua = _conf[suffix].useragent.custom; // is ALWAYS set

			channel.setRequestHeader("User-Agent", nua, false);
			return true;
		} catch (ex) {
			dump("_adjustUserAgent: " + ex);
		}
		return false;
	}

	function _adjustLanguage(channel, suffix) {
		try {	
			var ndel = _conf[suffix].language.mode;
			var nlang = _conf[suffix].language.custom;
			var oheader = false;
			var nheader;
			
			if (!ndel) {
				oheader = channel.getRequestHeader( 'Accept-Language' );
			}
			
			nheader = nlang + (oheader ? ","+oheader : "");
			
			channel.setRequestHeader("Accept-Language", nheader, false);
			return true;
		} catch (ex) {
			dump("_adjustLanguage: " + ex);
		}
		return false;
	}

	function dump(message) {
		// UNCOMMENT 4 DEBUGGING!
		//Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService).logStringMessage(message);
	}

	function _onAppStartup() {
		try {
			var observerService = Components.classes["@mozilla.org/observer-service;1"]
					.getService(Components.interfaces.nsIObserverService);
			observerService.addObserver(headercontrolEngine, "http-on-modify-request", true);

			var prefService = Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefService);
			_pref = prefService.getBranch("extensions.headercontrol.");
			_pref.QueryInterface(Components.interfaces.nsIPrefBranch2);
			_pref.addObserver("enabled", headercontrolEngine, true);
			_pref.addObserver("conf", headercontrolEngine, true);
		} catch (ex) {
			dump("onAppStartup: " + ex);
		}
	}

	function _onModifyRequest(channel) {
		dump("_onModifyRequest");
		try {
			var t;
			var i;
			var match;
			var rconf;

			channel.QueryInterface(Components.interfaces.nsIChannel);

			// find conf
			t = channel.URI.host.split(".");
			t = t.reverse();
			
			rconf = "";
			match = "";
			for (var i=0; i<t.length; i++) {
				if (_conf[match] != undefined) {
					rconf = match;
				}

				match += "." + t[i];
			}
			if (_conf[match] != undefined) {
				rconf = match;
			}

			// adjust request (default config is always found!)
			if (_conf[rconf].referer.enabled) {
				_adjustReferer(channel, rconf);
			}
			if (_conf[rconf].useragent.enabled) {
				_adjustUseragent(channel, rconf);
			}
			if (_conf[rconf].language.enabled) {
				_adjustLanguage(channel, rconf);
			}
		} catch (ex) {
			dump("_onModifyRequest: " + ex);
		}
	}

	function _onChangeEnabled() {
		try {
			_enabled = _pref.getBoolPref('enabled');
		} catch (ex) {
			dump("_onChangeEnabled: " + ex);
		}
	}

	return {
		classDescription: "HeaderControl component",  
		classID : Components.ID("{38003c5d-0123-4588-86d9-0b9e563d3d2c}"),
		contractID : "@torvin/headercontrolrevived;1",
		QueryInterface : XPCOMUtils.generateQI([
			Components.interfaces.nsISupports, 
			Components.interfaces.nsIObserver, 
			Components.interfaces.nsISupportsWeakReference
		]),

		// Implement nsIObserver
		observe: function(aSubject, aTopic, aData) {
			try {
				switch (aTopic) {
					case 'http-on-modify-request' :
						aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
						if (_enabled) {
							_onModifyRequest(aSubject);
						}
						break;

					case 'nsPref:changed' :
						aSubject.QueryInterface(Components.interfaces.nsIPrefBranch);
						switch (aData) {
							case 'enabled':
								_onChangeEnabled();
								break;
							case 'conf':
								_readConf();
								break;
							default:
								dump("observe: unknown pref changing: " + aData);
								break;
						}
						break;

					case 'profile-after-change' :
						_onAppStartup();
						_onChangeEnabled();
						_readConf();
						dump("HeaderControl started");
						break;

					default :
						dump("observe: unknown topic: " + aTopic);
						break;
				}
			} catch (ex) {
				dump("observe: " + ex);
			}
		},
	};
}();

HeaderControl.prototype = headercontrolEngine;

var components = [HeaderControl];  
if ("generateNSGetFactory" in XPCOMUtils)  
  var NSGetFactory = XPCOMUtils.generateNSGetFactory(components);  // Firefox 4.0 and higher  
else  
  var NSGetModule = XPCOMUtils.generateNSGetModule(components);    // Firefox 3.x  